@@ -10,6 +10,7 @@
 
 #include <bonobo/bonobo-moniker-util.h>
 #include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-event-source.h>
 #include <bonobo/bonobo-arg.h>
 
 #include "bonobo-config-subdir.h"
@@ -29,8 +30,30 @@ struct _BonoboConfigSubdirPrivate {
 	Bonobo_ConfigDatabase  db;
 	char                  *subdir;
 	char                  *moniker;
+	BonoboEventSource     *es;
 };
 
+static void
+notify_cb (BonoboListener    *listener,
+	   char              *event_name, 
+	   CORBA_any         *any,
+	   CORBA_Environment *ev,
+	   gpointer           user_data)
+{
+	BonoboConfigSubdir *sd = BONOBO_CONFIG_SUBDIR (user_data);
+	char *tmp, *ename;
+
+	tmp = bonobo_event_subtype (event_name);
+	ename = g_strconcat ("Bonobo/Property:change:", tmp, NULL); 
+	g_free (tmp);
+
+	printf ("GOT EV %s\n", event_name);
+
+	//bonobo_event_source_notify_listeners (cb->es, ename, any, NULL);
+
+	g_free (ename);
+}
+
 static gboolean
 server_broken (CORBA_Environment* ev)
 {
@@ -407,13 +430,17 @@ bonobo_config_proxy_new       (Bonobo_ConfigDatabase  db,
 {
 	BonoboConfigSubdir *sd;
 	CORBA_Environment ev;
-	char *dir = NULL;
+	char *m, *dir = NULL;
 	int l;
 
 	g_return_val_if_fail (db != CORBA_OBJECT_NIL, CORBA_OBJECT_NIL);
 
 	if (subdir && subdir [0]) {
-		dir = g_strdup (subdir);
+
+		if (subdir [0] == '/')
+			dir = g_strdup (subdir);
+		else
+			dir = g_strconcat ("/", subdir, NULL);
 
 		while ((l = strlen (dir)) > 1 && dir [l - 1] == '/') 
 			dir [l] = '\0';
@@ -431,6 +458,10 @@ bonobo_config_proxy_new       (Bonobo_ConfigDatabase  db,
 
 	sd->priv->subdir = dir;
 	sd->priv->moniker = g_strdup (moniker);
+	sd->priv->es = bonobo_event_source_new ();
+
+	bonobo_object_add_interface (BONOBO_OBJECT (sd), 
+				     BONOBO_OBJECT (sd->priv->es));
 
 	CORBA_exception_init (&ev);
 
@@ -446,6 +477,12 @@ bonobo_config_proxy_new       (Bonobo_ConfigDatabase  db,
 
 	sd->priv->db = db;
 	
+	m = g_strconcat ("Bonobo/ConfigDatabase:change:", NULL);
+	
+	bonobo_event_source_client_add_listener (db, notify_cb, m, NULL, sd);
+
+	g_free (m);
+
 	return CORBA_Object_duplicate (BONOBO_OBJREF (sd), NULL);
 }
 
